@@ -189,7 +189,7 @@ public:
     QString versionFixedIn() const {
         return getData("cf_versionfixedin");
     }
-    
+
     void setValid(bool valid) {
         m_isValid = valid;
     }
@@ -246,7 +246,7 @@ private:
 class BugReportXMLParser
 {
 public:
-    BugReportXMLParser(const QByteArray &);
+    explicit BugReportXMLParser(const QByteArray &);
 
     BugReport parse();
 
@@ -264,7 +264,7 @@ private:
 class BugListCSVParser
 {
 public:
-    BugListCSVParser(const QByteArray&);
+    explicit BugListCSVParser(const QByteArray&);
 
     bool isValid() const {
         return m_isValid;
@@ -282,7 +282,7 @@ class BugzillaManager : public QObject
     Q_OBJECT
 
 public:
-    BugzillaManager(QObject *parent = 0);
+    explicit BugzillaManager(QObject *parent = 0);
 
     /* Login methods */
     void tryLogin();
@@ -296,23 +296,23 @@ public:
 
     void searchBugs(const QStringList & products, const QString & severity,
                     const QString & date_start, const QString & date_end , QString comment);
-    
+
     void sendReport(const BugReport & report);
-    
+
     void attachTextToReport(const QString & text, const QString & filename,
                             const QString & description, int bugId, const QString & comment);
 
     void addMeToCC(int bugId);
 
     void checkVersionsForProduct(const QString &);
-    
+
     /* Misc methods */
     QString urlForBug(int bug_number) const;
 
     void setCustomBugtrackerUrl(const QString &);
 
     void stopCurrentSearch();
-    
+
 private Q_SLOTS:
     /* Slots to handle KJob::finished */
     void fetchBugJobFinished(KJob*);
