@@ -1562,6 +1562,7 @@ sftpProtocol::StatusCode sftpProtocol::sftpCopyGet(const KUrl& url, const QStrin
     offset = seekPos(fd, 0, SEEK_END);
     if(offset < 0) {
       errorCode = ERR_CANNOT_RESUME;
+      ::close(fd);
       return sftpProtocol::ClientError;                            // client side error
     }
     kDebug(KIO_SFTP_DB) << "resuming at" << offset;
@@ -1633,7 +1634,9 @@ sftpProtocol::StatusCode sftpProtocol::sftpCopyPut(const KUrl& url, const QStrin
   totalSize(buff.st_size);
 
   // delegate the real work (errorCode gets status) ...
-  return sftpPut(url, permissions, flags, errorCode, fd);
+  StatusCode ret = sftpPut(url, permissions, flags, errorCode, fd);
+  ::close(fd);
+  return ret;
 }
 
 
@@ -2040,7 +2043,7 @@ bool sftpProtocol::GetRequest::enqueueChunks() {
 
     pendingRequests.enqueue(request);
 
-    if (mFile->offset > mSb->size) {
+    if (mFile->offset >= mSb->size) {
       // Do not add any more chunks if the offset is larger than the given file size.
       // However this is done after adding a request as the remote file size may
       // have changed in the meantime.
@@ -2076,10 +2079,8 @@ int sftpProtocol::GetRequest::readChunks(QByteArray &data) {
     if (bytesread == 0 || bytesread == SSH_AGAIN) {
       if (bytesread == 0) {
         pendingRequests.dequeue();
-      } else {
-        // Decrease maximum pending requests as we did not receive data fast enough
-        mMaxPendingRequests = qMax(1, mMaxPendingRequests / 2);
       }
+
       // Done reading or timeout
       data.resize(data.size() - request.expectedLength);
       break;
@@ -2116,9 +2117,6 @@ int sftpProtocol::GetRequest::readChunks(QByteArray &data) {
     pendingRequests.dequeue();
   }
 
-  // Adjust maximum pending requests
-  mMaxPendingRequests = qMin(mMaxPendingRequests * 2, MAX_TRANSFER_SIZE / MAX_XFER_BUF_SIZE);
-
   return totalRead;
 }
 
